unit MeuEdit;

interface

uses Classes, SysUtils, Graphics, StdCtrls, Controls, Messages, Windows;

type
  TDataType = (ddTexto, ddNumero, ddCPF, ddCNPJ);

type
  TMeuEdit = Class(TEdit)

  private
    FDados        : TDataType;
    FValor        : Extended;
    FAlignment    : TAlignment;
    FColorFocus   : TColor;
    FCasaDecimal  : Integer;
    FColorUnFocus : TColor;
    procedure SetAlignment(const Value: TAlignment);
    procedure SetColorFocus(const Value: TColor);
    procedure SetColorUnFocus(const Value: TColor);
    procedure SetDados(const Value: TDataType);
    procedure SetCasaDecimal(const Value: Integer);
    procedure SetValor(const Value: Extended);
    function  GetValor: Extended;
  private
    procedure AjustaCor;
    procedure AjustaTexto;
  protected
    procedure CreateParams(var Params: TCreateParams); override;
    procedure DoExit; override;
    procedure DoEnter; override;
    procedure WMChar(var Message: TWMChar); message WM_CHAR;
  public
    function       CNPJ(num: string): boolean;
    function       CPF(num: string): Boolean;
    constructor    Create(AOwner: TComponent); override;
    class function TextoParaNumero(Valor: String): Extended;
  published
    property ColorFocus   : TColor     read FColorFocus   write SetColorFocus   default clWindow;
    property ColorUnFocus : TColor     read FColorUnFocus write SetColorUnFocus default clInfoBk;
    property Alignment    : TAlignment read FAlignment    write SetAlignment    default taLeftJustify;
    property Dados        : TDataType     read FDados        write SetDados        default ddTexto;
    property CasaDecimal  : Integer    read FCasaDecimal  write SetCasaDecimal  default 2;
    property Valor        : Extended   read GetValor      write SetValor;
  end;

procedure Register;

implementation

procedure Register;
begin
  RegisterComponents('Meus Componentes', [TMeuEdit]);
end;

function TMeuEdit.CPF(num: string): Boolean;
var
  n1,n2,n3,n4,n5,n6,n7,n8,n9:integer;
  d1,d2:integer;
  Digitado, Calculado: String;

begin
  n1 := StrToInt(num[1]);
  n2 := StrToInt(num[2]);
  n3 := StrToInt(num[3]);
  n4 := StrToInt(num[4]);
  n5 := StrToInt(num[5]);
  n6 := StrToInt(num[6]);
  n7 := StrToInt(num[7]);
  n8 := StrToInt(num[8]);
  n9 := StrToInt(num[9]);
  d1 := n9*2+n8*3+n7*4+n6*5+n5*6+n4*7+n3*8+n2*9+n1*10;
  d1 := 11-(d1 mod 11);
  if d1 >= 10 then
    d1 :=0;
  d2 := d1*2+n9*3+n8*4+n7*5+n6*6+n5*7+n4*8+n3*9+n2*10+n1*11;
  d2 := 11-(d2 mod 11);
  if d2>=10 then
    d2 :=0;
  Calculado := inttostr(d1)+inttostr(d2);
  Digitado := num[10]+num[11];
  if Calculado = digitado then
    CPF := True
  else
    CPF := False;
end;

function TMeuEdit.CNPJ(num: string): boolean;
var
   n1,  n2,   n3,
   n4,  n5,   n6,
   n7,  n8,   n9,
   n10, n11,
   n12, d1, d2        : Integer;
  Digitado, Calculado : String;

begin
  n1  := StrToInt(num[1]);
  n2  := StrToInt(num[2]);
  n3  := StrToInt(num[3]);
  n4  := StrToInt(num[4]);
  n5  := StrToInt(num[5]);
  n6  := StrToInt(num[6]);
  n7  := StrToInt(num[7]);
  n8  := StrToInt(num[8]);
  n9  := StrToInt(num[9]);
  n10 := StrToInt(num[10]);
  n11 := StrToInt(num[11]);
  n12 := StrToInt(num[12]);
  d1  := n12*2+n11*3+n10*4+n9*5+n8*6+n7*7+n6*8+n5*9+n4*2+n3*3+n2*4+n1*5;
  d1  := 11-(d1 mod 11);
  if d1>= 10 then
    d1 := 0;
  d2 := d1*2+ n12*3+n11*4+n10*5+n9*6+n8*7+n7*8+n6*9+n5*2+n4*3+n3*4+n2*5+n1*6;
  d2 := 11-(d2 mod 11);
  if d2>=10 then
    d2 := 0;
  calculado := IntToStr(d1) + inttostr(d2);
  digitado  := num[13]+num[14];
  if calculado = digitado then
    CNPJ := True
  else
    CNPJ := False;
end;

procedure TMeuEdit.AjustaCor;
begin
  if Focused then
    Color := FColorFocus
  else
    Color := FColorUnFocus;
end;

procedure TMeuEdit.AjustaTexto;
var
  sAuxiliar,
  sDecimal : String;

begin

  if Trim(Text) = EmptyStr then
    Exit;
  case Dados of
    ddCPF :
      begin
        if not CPF(Text) then
          Raise Exception.Create('CPF invlido.')
        else
        begin
          Text      := StringOfChar('0', 11 - Length(Text)) + Text;
          sAuxiliar := Copy(Text, 1, 3) + '.' +
                       Copy(Text, 4, 3) + '.' +
                       Copy(Text, 7, 3) + '-' +
                       Copy(Text, 10, 2);
          Text      := sAuxiliar;
        end;
      end;
    ddCNPJ :
      begin
        if not CNPJ(Text) then
          Raise Exception.Create('CNPJ invlido.')
        else
        begin
          Text      := StringOfChar('0', 14 - Length(Text)) + Text;
          sAuxiliar := Copy(Text, 1, 2) + '.' +
                       Copy(Text, 3, 3) + '.' +
                       Copy(Text, 6, 3) + '/' +
                       Copy(Text, 9, 4) + '-' +
                       Copy(Text, 12, 2);
          Text      := sAuxiliar;
        end;
      end;
    ddNumero :
      begin
        FValor   := TextoParaNumero(Text);
        if CasaDecimal > 0 then
        begin
          sDecimal := StringOfChar('0', CasaDecimal);
          Text     := FormatFloat(',0.' + sDecimal, FValor);
        end
        else
          Text := FormatFloat(',0', FValor);
      end;
  end;

end;

constructor TMeuEdit.Create(AOwner: TComponent);
begin
  inherited;
  FAlignment    := taLeftJustify;
  FColorFocus   := clWindow;
  FColorUnFocus := clInfoBk;
  FAlignment    := taLeftJustify;
  FDados        := ddTexto;
  FCasaDecimal  := 2;
  FValor        := 0;
end;

procedure TMeuEdit.CreateParams(var Params: TCreateParams);
begin
  inherited CreateParams(Params);
  case Alignment of
    taLeftJustify  : Params.Style := Params.Style or LongWord(ES_Left);
    taRightJustify : Params.Style := Params.Style or LongWord(ES_Right);
    taCenter       : Params.Style := Params.Style or LongWord(ES_Center);
  end;
  AjustaCor;
  AjustaTexto;
end;

procedure TMeuEdit.DoEnter;
begin
  inherited DoEnter;
  AjustaCor;
  case Dados of
    ddCPF,
    ddCNPJ   :
      begin
        Text := StringReplace(Text, ThousandSeparator, EmptyStr, [rfReplaceAll]);
        Text := StringReplace(Text, Chr(VK_HELP),      EmptyStr, [rfReplaceAll]);
        Text := StringReplace(Text, Chr(VK_INSERT),    EmptyStr, [rfReplaceAll]);
      end;
    ddNumero : Text := StringReplace(Text, ThousandSeparator, EmptyStr, [rfReplaceAll]);
  end;
end;

procedure TMeuEdit.DoExit;
begin
  inherited DoExit;
  AjustaCor;
  AjustaTexto;
end;

function TMeuEdit.GetValor: Extended;
begin
  Result := FValor;
  AjustaTexto;
end;

procedure TMeuEdit.SetAlignment(const Value: TAlignment);
begin
  if FAlignment <> Value then
  begin
    FAlignment := Value;
    RecreateWnd;
  end;
  AjustaCor;
end;

procedure TMeuEdit.SetCasaDecimal(const Value: Integer);
begin
  FCasaDecimal := Value;
  AjustaTexto;
  RecreateWnd;
end;

procedure TMeuEdit.SetColorFocus(const Value: TColor);
begin
  FColorFocus := Value;
  RecreateWnd;
end;

procedure TMeuEdit.SetColorUnFocus(const Value: TColor);
begin
  FColorUnFocus := Value;
  RecreateWnd;
end;

procedure TMeuEdit.SetDados(const Value: TDataType);
begin
  FDados := Value;
  case FDados of
    ddCPF,
    ddCNPJ   : Text := EmptyStr;
    ddNumero : FAlignment := taRightJustify;
  else
    FAlignment := taLeftJustify;
  end;
  AjustaTexto;
end;

procedure TMeuEdit.SetValor(const Value: Extended);
begin
  FValor := Value;
  AjustaTexto;
end;

class function TMeuEdit.TextoParaNumero(Valor: String): Extended;
var
  nIndex : Integer;
  sTexto : String;

begin
  sTexto := '0';
  for nIndex := 1 to Length(Valor) do
  begin
    if Valor[nIndex] in ['0'..'9', DecimalSeparator] then
      sTexto := sTexto + Valor[nIndex];
  end;
  Result := StrToFloat(sTexto);
end;

procedure TMeuEdit.WMChar(var Message: TWMChar);
var
  bValido  : Boolean;
  nPosicao : Integer;

begin
  bValido := False;
  case Dados of
    ddNumero :
      begin
        if Chr(Message.CharCode) in ['0'..'9', DecimalSeparator, #8, #9, #13] then
        begin
          if Chr(Message.CharCode) in [#8, #9, #13] then
            bValido := True
          else
          begin
            //Verifica se  um decimal separador, se for, no permiti digitar duas vezes o separador decimal;
            if (Chr(Message.CharCode) = DecimalSeparator) then
            begin
              if (Pos(DecimalSeparator, Text)=0) then
                bValido := True;
            end
            else
            begin
              nPosicao := Pos(DecimalSeparator, Text);
              if (nPosicao > 0) then
                if (SelStart < (nPosicao + CasaDecimal)) then
                  if ((Length(Text) < nPosicao + CasaDecimal) or
                     ((SelStart < Length(Text))) and (SelStart < nPosicao)) then
                    bValido := True
                  else
                else
              else
                bValido := True;
            end;
          end;
        end;
      end;
    ddCPF,
    ddCNPJ :
      begin
        if Chr(Message.CharCode) in ['0'..'9', #8, #9, #13] then
          bValido := True;
      end;
  else
    bValido := True;
  end;
  if bValido then
    inherited;
end;

end.
